# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: types.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'types.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0btypes.proto\x12\x15io.statefun.sdk.types\"\x1f\n\x0e\x42ooleanWrapper\x12\r\n\x05value\x18\x01 \x01(\x08\"\x1b\n\nIntWrapper\x12\r\n\x05value\x18\x01 \x01(\x0f\"\x1d\n\x0c\x46loatWrapper\x12\r\n\x05value\x18\x01 \x01(\x02\"\x1c\n\x0bLongWrapper\x12\r\n\x05value\x18\x01 \x01(\x10\"\x1e\n\rDoubleWrapper\x12\r\n\x05value\x18\x01 \x01(\x01\"\x1e\n\rStringWrapper\x12\r\n\x05value\x18\x01 \x01(\tB=\n-org.apache.flink.statefun.sdk.types.generatedP\x01Z\n.;protocolb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'types_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n-org.apache.flink.statefun.sdk.types.generatedP\001Z\n.;protocol'
  _globals['_BOOLEANWRAPPER']._serialized_start=38
  _globals['_BOOLEANWRAPPER']._serialized_end=69
  _globals['_INTWRAPPER']._serialized_start=71
  _globals['_INTWRAPPER']._serialized_end=98
  _globals['_FLOATWRAPPER']._serialized_start=100
  _globals['_FLOATWRAPPER']._serialized_end=129
  _globals['_LONGWRAPPER']._serialized_start=131
  _globals['_LONGWRAPPER']._serialized_end=159
  _globals['_DOUBLEWRAPPER']._serialized_start=161
  _globals['_DOUBLEWRAPPER']._serialized_end=191
  _globals['_STRINGWRAPPER']._serialized_start=193
  _globals['_STRINGWRAPPER']._serialized_end=223
# @@protoc_insertion_point(module_scope)
