# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: request-reply.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'request-reply.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13request-reply.proto\x12\x18io.statefun.sdk.reqreply\"6\n\x07\x41\x64\x64ress\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\n\n\x02id\x18\x03 \x01(\t\"@\n\nTypedValue\x12\x10\n\x08typename\x18\x01 \x01(\t\x12\x11\n\thas_value\x18\x02 \x01(\x08\x12\r\n\x05value\x18\x03 \x01(\x0c\"\x9c\x04\n\nToFunction\x12Q\n\ninvocation\x18\x64 \x01(\x0b\x32;.io.statefun.sdk.reqreply.ToFunction.InvocationBatchRequestH\x00\x1a_\n\x0ePersistedValue\x12\x12\n\nstate_name\x18\x01 \x01(\t\x12\x39\n\x0bstate_value\x18\x02 \x01(\x0b\x32$.io.statefun.sdk.reqreply.TypedValue\x1aw\n\nInvocation\x12\x31\n\x06\x63\x61ller\x18\x01 \x01(\x0b\x32!.io.statefun.sdk.reqreply.Address\x12\x36\n\x08\x61rgument\x18\x02 \x01(\x0b\x32$.io.statefun.sdk.reqreply.TypedValue\x1a\xd5\x01\n\x16InvocationBatchRequest\x12\x31\n\x06target\x18\x01 \x01(\x0b\x32!.io.statefun.sdk.reqreply.Address\x12\x42\n\x05state\x18\x02 \x03(\x0b\x32\x33.io.statefun.sdk.reqreply.ToFunction.PersistedValue\x12\x44\n\x0binvocations\x18\x03 \x03(\x0b\x32/.io.statefun.sdk.reqreply.ToFunction.InvocationB\t\n\x07request\"\xbc\r\n\x0c\x46romFunction\x12V\n\x11invocation_result\x18\x64 \x01(\x0b\x32\x39.io.statefun.sdk.reqreply.FromFunction.InvocationResponseH\x00\x12k\n\x1dincomplete_invocation_context\x18\x65 \x01(\x0b\x32\x42.io.statefun.sdk.reqreply.FromFunction.IncompleteInvocationContextH\x00\x1a\xf2\x01\n\x16PersistedValueMutation\x12\x61\n\rmutation_type\x18\x01 \x01(\x0e\x32J.io.statefun.sdk.reqreply.FromFunction.PersistedValueMutation.MutationType\x12\x12\n\nstate_name\x18\x02 \x01(\t\x12\x39\n\x0bstate_value\x18\x03 \x01(\x0b\x32$.io.statefun.sdk.reqreply.TypedValue\"&\n\x0cMutationType\x12\n\n\x06\x44\x45LETE\x10\x00\x12\n\n\x06MODIFY\x10\x01\x1aw\n\nInvocation\x12\x31\n\x06target\x18\x01 \x01(\x0b\x32!.io.statefun.sdk.reqreply.Address\x12\x36\n\x08\x61rgument\x18\x02 \x01(\x0b\x32$.io.statefun.sdk.reqreply.TypedValue\x1a\xd0\x01\n\x11\x44\x65layedInvocation\x12\x1f\n\x17is_cancellation_request\x18\n \x01(\x08\x12\x1a\n\x12\x63\x61ncellation_token\x18\x0b \x01(\t\x12\x13\n\x0b\x64\x65lay_in_ms\x18\x01 \x01(\x03\x12\x31\n\x06target\x18\x02 \x01(\x0b\x32!.io.statefun.sdk.reqreply.Address\x12\x36\n\x08\x61rgument\x18\x03 \x01(\x0b\x32$.io.statefun.sdk.reqreply.TypedValue\x1av\n\rEgressMessage\x12\x18\n\x10\x65gress_namespace\x18\x01 \x01(\t\x12\x13\n\x0b\x65gress_type\x18\x02 \x01(\t\x12\x36\n\x08\x61rgument\x18\x03 \x01(\x0b\x32$.io.statefun.sdk.reqreply.TypedValue\x1a\xe2\x02\n\x12InvocationResponse\x12V\n\x0fstate_mutations\x18\x01 \x03(\x0b\x32=.io.statefun.sdk.reqreply.FromFunction.PersistedValueMutation\x12L\n\x11outgoing_messages\x18\x02 \x03(\x0b\x32\x31.io.statefun.sdk.reqreply.FromFunction.Invocation\x12U\n\x13\x64\x65layed_invocations\x18\x03 \x03(\x0b\x32\x38.io.statefun.sdk.reqreply.FromFunction.DelayedInvocation\x12O\n\x11outgoing_egresses\x18\x04 \x03(\x0b\x32\x34.io.statefun.sdk.reqreply.FromFunction.EgressMessage\x1a\xb8\x01\n\x0e\x45xpirationSpec\x12N\n\x04mode\x18\x01 \x01(\x0e\x32@.io.statefun.sdk.reqreply.FromFunction.ExpirationSpec.ExpireMode\x12\x1b\n\x13\x65xpire_after_millis\x18\x02 \x01(\x03\"9\n\nExpireMode\x12\x08\n\x04NONE\x10\x00\x12\x0f\n\x0b\x41\x46TER_WRITE\x10\x01\x12\x10\n\x0c\x41\x46TER_INVOKE\x10\x02\x1a\x8f\x01\n\x12PersistedValueSpec\x12\x12\n\nstate_name\x18\x01 \x01(\t\x12N\n\x0f\x65xpiration_spec\x18\x02 \x01(\x0b\x32\x35.io.statefun.sdk.reqreply.FromFunction.ExpirationSpec\x12\x15\n\rtype_typename\x18\x03 \x01(\t\x1ap\n\x1bIncompleteInvocationContext\x12Q\n\x0emissing_values\x18\x01 \x03(\x0b\x32\x39.io.statefun.sdk.reqreply.FromFunction.PersistedValueSpecB\n\n\x08responseB@\n0org.apache.flink.statefun.sdk.reqreply.generatedP\x01Z\n.;protocolb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'request_reply_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n0org.apache.flink.statefun.sdk.reqreply.generatedP\001Z\n.;protocol'
  _globals['_ADDRESS']._serialized_start=49
  _globals['_ADDRESS']._serialized_end=103
  _globals['_TYPEDVALUE']._serialized_start=105
  _globals['_TYPEDVALUE']._serialized_end=169
  _globals['_TOFUNCTION']._serialized_start=172
  _globals['_TOFUNCTION']._serialized_end=712
  _globals['_TOFUNCTION_PERSISTEDVALUE']._serialized_start=269
  _globals['_TOFUNCTION_PERSISTEDVALUE']._serialized_end=364
  _globals['_TOFUNCTION_INVOCATION']._serialized_start=366
  _globals['_TOFUNCTION_INVOCATION']._serialized_end=485
  _globals['_TOFUNCTION_INVOCATIONBATCHREQUEST']._serialized_start=488
  _globals['_TOFUNCTION_INVOCATIONBATCHREQUEST']._serialized_end=701
  _globals['_FROMFUNCTION']._serialized_start=715
  _globals['_FROMFUNCTION']._serialized_end=2439
  _globals['_FROMFUNCTION_PERSISTEDVALUEMUTATION']._serialized_start=929
  _globals['_FROMFUNCTION_PERSISTEDVALUEMUTATION']._serialized_end=1171
  _globals['_FROMFUNCTION_PERSISTEDVALUEMUTATION_MUTATIONTYPE']._serialized_start=1133
  _globals['_FROMFUNCTION_PERSISTEDVALUEMUTATION_MUTATIONTYPE']._serialized_end=1171
  _globals['_FROMFUNCTION_INVOCATION']._serialized_start=1173
  _globals['_FROMFUNCTION_INVOCATION']._serialized_end=1292
  _globals['_FROMFUNCTION_DELAYEDINVOCATION']._serialized_start=1295
  _globals['_FROMFUNCTION_DELAYEDINVOCATION']._serialized_end=1503
  _globals['_FROMFUNCTION_EGRESSMESSAGE']._serialized_start=1505
  _globals['_FROMFUNCTION_EGRESSMESSAGE']._serialized_end=1623
  _globals['_FROMFUNCTION_INVOCATIONRESPONSE']._serialized_start=1626
  _globals['_FROMFUNCTION_INVOCATIONRESPONSE']._serialized_end=1980
  _globals['_FROMFUNCTION_EXPIRATIONSPEC']._serialized_start=1983
  _globals['_FROMFUNCTION_EXPIRATIONSPEC']._serialized_end=2167
  _globals['_FROMFUNCTION_EXPIRATIONSPEC_EXPIREMODE']._serialized_start=2110
  _globals['_FROMFUNCTION_EXPIRATIONSPEC_EXPIREMODE']._serialized_end=2167
  _globals['_FROMFUNCTION_PERSISTEDVALUESPEC']._serialized_start=2170
  _globals['_FROMFUNCTION_PERSISTEDVALUESPEC']._serialized_end=2313
  _globals['_FROMFUNCTION_INCOMPLETEINVOCATIONCONTEXT']._serialized_start=2315
  _globals['_FROMFUNCTION_INCOMPLETEINVOCATIONCONTEXT']._serialized_end=2427
# @@protoc_insertion_point(module_scope)
